﻿using System.Collections.Generic;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Navigation;
using System.Threading.Tasks;
using Microsoft.Live;
using Windows.UI.Popups;
using System;
using Microsoft.WindowsAzure.MobileServices;

namespace VSMMobileServiceClient
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class MainPage : Page
    {
        List<ContactItem> _contacts;
        ContactItem _item;
        bool _update = false;
        LiveConnectSession _liveConnectSession;
        const string _azureMobileServiceEndpoint = "Your Azure Mobile Service Hostname";
        LiveConnectUser _user;
        LiveAuthClient _liveAuthClient;

        public MainPage()
        {
            this.InitializeComponent();
            _liveAuthClient = new LiveAuthClient(_azureMobileServiceEndpoint);
            ClearItem();
        }

        private async void ClearItem()
        {
            _item = new ContactItem();
            Contact.DataContext = _item;
            Delete.IsEnabled = false;
            _update = false;
            await UpdateStatus(string.Empty);
        }

        private async Task UpdateStatus(string status)
        {
            Status.Text = status;
            await Task.Delay(250);
        }

        private async Task Login()
        {
            while (_liveConnectSession == null)
            {
                LiveLoginResult liveLoginResult = await _liveAuthClient.LoginAsync(new[] { "wl.basic" });
                if (liveLoginResult.Status == LiveConnectSessionStatus.Connected)
                {
                    _liveConnectSession = liveLoginResult.Session;
                    LiveConnectClient liveConnectClient = new LiveConnectClient(liveLoginResult.Session);
                    LiveOperationResult liveOpResult = await liveConnectClient.GetAsync("me");
                    MobileServiceUser mobileServiceLoginResult = await App.MobileService.LoginAsync(liveLoginResult.Session.AuthenticationToken);
                    _user = new LiveConnectUser(mobileServiceLoginResult.UserId, liveOpResult.Result["first_name"].ToString(),
                        liveOpResult.Result["last_name"].ToString());
                    MobileUser.DataContext = _user;
                    SignOut.IsEnabled = _liveAuthClient.CanLogout;
                    Contact.Visibility = Windows.UI.Xaml.Visibility.Visible;
                }
                else
                {
                    _liveConnectSession = null;
                    var loginErrorDialog = new MessageDialog("Please try loggin in again.", "Retry Login");
                    var okCommand = new UICommand("OK");
                    var cancelCommand = new UICommand("Cancel");
                    loginErrorDialog.Commands.Add(okCommand);
                    var dialogResult = await loginErrorDialog.ShowAsync();
                    if (dialogResult != new UICommand("OK"))
                    {
                        Application.Current.Exit();
                    }
                }
            }
        }

        /// <summary>
        /// Invoked when this page is about to be displayed in a Frame.
        /// </summary>
        /// <param name="e">Event data that describes how this page was reached.  The Parameter
        /// property is typically used to configure the page.</param>
        protected async override void OnNavigatedTo(NavigationEventArgs e)
        {
            await Login();
            await GetContacts();
        }

        private async Task GetContacts()
        {
            _contacts = await App.MobileService.GetTable<ContactItem>().ToListAsync();
            Contacts.ItemsSource = _contacts;
        }

        private async void Save_Click(object sender, RoutedEventArgs e)
        {
            if (_update)
                await App.MobileService.GetTable<ContactItem>().UpdateAsync(_item);
            else
                await App.MobileService.GetTable<ContactItem>().InsertAsync(_item);

            await UpdateStatus("Saved");
            await GetContacts();
            ClearItem();
        }

        private void Contacts_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            var selectedItem = (sender as ListBox).SelectedItem as ContactItem;
            if (selectedItem != null)
            {
                _item = selectedItem;
                Contact.DataContext = _item;
                _update = true;
                Delete.IsEnabled = true;
            }
        }

        private async void Delete_Click(object sender, RoutedEventArgs e)
        {
            await App.MobileService.GetTable<ContactItem>().DeleteAsync(_item);
            await UpdateStatus("Deleted");
            ClearItem();
            await GetContacts();
        }

        private void SignOut_Click(object sender, RoutedEventArgs e)
        {
            if (_liveAuthClient.CanLogout)
            {
                _liveAuthClient.Logout();
                SignIn.Visibility = Windows.UI.Xaml.Visibility.Visible;
                Contact.Visibility = Windows.UI.Xaml.Visibility.Collapsed;
            }
        }

        private async void SignIn_Click(object sender, RoutedEventArgs e)
        {
            await Login();
            SignIn.Visibility = Windows.UI.Xaml.Visibility.Collapsed;
        }
    }
}
